# -*- coding: utf-8 -*-

import sys
import threading
from os import path

from PySide import QtCore
from PySide.QtCore import Qt
from PySide.QtGui import *

import main

app = QApplication(sys.argv)

class Gui(QTabWidget):

    curdir = sys.executable[::-1].split('\\', 1)[1][::-1] if '\\' in sys.executable else sys.executable[::-1].split('/', 1)[1][::-1]
    # curdir = ''

    updateStatusSignalDatal = QtCore.Signal(str)
    updateStatusSignalIFC = QtCore.Signal(str)

    def __init__(self):
        super(Gui, self).__init__()

        self.updateStatusSignalDatal.connect(self.onUpdateStatusDatal)
        self.updateStatusSignalIFC.connect(self.onUpdateStatusIFC)

        self._progressBarDatal = QProgressBar()
        self._progressBarIFC = QProgressBar()

        self._progressBarDatal.setTextVisible(False)
        self._progressBarIFC.setTextVisible(False)

        self.resize(400, 400)
        self.setWindowTitle("Converter")

        self.__createUI()

    ################################################################################################################################################################################

    @QtCore.Slot(str)
    def onUpdateStatusDatal(self, ret):
        self._progressBarDatal.setVisible(False)
        self._DatalStartButton.setEnabled(True)
        self.showDialog(ret)

    @QtCore.Slot(str)
    def onUpdateStatusIFC(self, ret):
        self._progressBarIFC.setVisible(False)
        self._IFCStartButton.setEnabled(True)
        self.showDialog(ret)

    ################################################################################################################################################################################

    def __createUI(self):
        self.addTab(self._addIFC(), 'IFC')
        self.addTab(self._addDatal(), 'Datal')
        self.addTab(self._addDatabase(), 'Table')

    def _addIFC(self):
        self._IFCTab = QWidget()

        IFCLayout = QVBoxLayout()

        self._IFCStartButton = QPushButton('Start')
        self._IFCStartButton.setDisabled(True)

        IFCBox = QHBoxLayout()
        IFCLabel = QLabel('IFC:')
        IFCText = QLineEdit()
        IFCPushButton = QPushButton('...')

        IFCPushButton.setFixedWidth(22)
        IFCPushButton.clicked.connect(
            lambda:(
                IFCText.setText(QFileDialog.getOpenFileName(self, "IFC File", filter='*.ifc')[0]),
                self._IFCStartButton.setDisabled(False)
            )
        )

        self._IFCStartButton.setFixedWidth(80)
        self._IFCStartButton.clicked.connect(
            lambda:(
                self._IFCStartButton.setEnabled(False),
                self.convertIFC(str(IFCText.text()))
            )
        )

        IFCBox.addWidget(IFCLabel)
        IFCBox.addWidget(IFCText)
        IFCBox.addWidget(IFCPushButton)

        IFCLayout.addLayout(IFCBox)
        IFCLayout.addWidget(self._progressBarIFC)
        IFCLayout.addWidget(self._IFCStartButton)

        self._progressBarIFC.setVisible(False)

        self._IFCTab.setLayout(IFCLayout)

        return self._IFCTab

    def _addDatal(self):

        self._DatalTab = QWidget()

        DatalLayout = QVBoxLayout()

        self._DatalStartButton = QPushButton('Start')
        self._DatalStartButton.setDisabled(True)

        DatalBox = QHBoxLayout()
        Datallabel = QLabel('Datal:')
        DatalText = QLineEdit()
        DatalPushButton = QPushButton('...')

        DatalPushButton.setFixedWidth(22)
        DatalPushButton.clicked.connect(
            lambda:(
                DatalText.setText(QFileDialog.getOpenFileName(self, "Datal File", filter='*.datal *.txt *.dtl')[0]),
                self._DatalStartButton.setDisabled(False)
            )
        )

        IFCBox = QHBoxLayout()
        IFCLabel = QLabel('IFC Template:')
        IFCText = QLineEdit()
        IFCPushButton = QPushButton('...')

        IFCPushButton.setFixedWidth(22)
        IFCPushButton.clicked.connect(
            lambda:(
                IFCText.setText(QFileDialog.getOpenFileName(self, "IFC File", filter='*.ifc')[0]),
            )
        )

        self._DatalStartButton.setFixedWidth(80)
        self._DatalStartButton.clicked.connect(
            lambda:(
                self._DatalStartButton.setEnabled(False),
                self.convertDatal(str(IFCText.text()) , str(DatalText.text())),
            )
        )

        DatalBox.addWidget(Datallabel)
        DatalBox.addWidget(DatalText)
        DatalBox.addWidget(DatalPushButton)

        IFCBox.addWidget(IFCLabel)
        IFCBox.addWidget(IFCText)
        IFCBox.addWidget(IFCPushButton)

        DatalLayout.addLayout(DatalBox)
        DatalLayout.addLayout(IFCBox)
        DatalLayout.addWidget(self._progressBarDatal)
        DatalLayout.addWidget(self._DatalStartButton)

        self._progressBarDatal.setVisible(False)

        self._DatalTab.setLayout(DatalLayout)

        return self._DatalTab

    def _addDatabase(self):
        self._DBTab = QWidget()

        DBBox = QHBoxLayout()
        DBlabel = QLabel('Table of Norm:')
        self._DBText = QLineEdit(path.join(self.curdir, 'resources\\AISC_Shapes_Database.csv'))
        DBPushButton = QPushButton('...')

        DBPushButton.setFixedWidth(22)
        DBPushButton.clicked.connect(
            lambda:(
                self._DBText.setText(QFileDialog.getOpenFileName(self, "CSV File", filter='*.csv *.txt')[0]),
            )
        )

        DBBox.addWidget(DBlabel)
        DBBox.addWidget(self._DBText)
        DBBox.addWidget(DBPushButton)

        self._DBTab.setLayout(DBBox)

        return self._DBTab

    ################################################################################################################################################################################

    def convertIFC(self, IFCText):
        self._progressBarIFC.setVisible(True)
        self._progressBarIFC.setMaximum(0)
        self._progressBarIFC.setMinimum(0)
        self._progressBarIFC.setValue(0)

        t = threading.Thread(target=self._convertIFC, args=[IFCText])
        t.start()

    def _convertIFC(self, IFCText):
        ret = main.main(IFCText, DBInput=str(self._DBText.text()), curdir=self.curdir)
        self.updateStatusSignalIFC.emit(ret)

    def convertDatal(self, IFCText, DatalText):
        self._progressBarDatal.setVisible(True)
        self._progressBarDatal.setMaximum(0)
        self._progressBarDatal.setMinimum(0)
        self._progressBarDatal.setValue(0)
        # print IFCText
        t = threading.Thread(target=self._convertDatal, args=[IFCText, DatalText])
        t.start()

    def _convertDatal(self, IFCText, DatalText):
        ret = main.main(IFCText, DatalText, str(self._DBText.text()), self.curdir)
        self.updateStatusSignalDatal.emit(ret)

    ################################################################################################################################################################################

    @staticmethod
    def showDialog(funcReturn):

        dialog = QDialog(None, Qt.WindowSystemMenuHint | Qt.WindowTitleHint)
        dialog.setMinimumWidth(100)

        vBox = QVBoxLayout()

        if funcReturn == 'OK':
            label = QLabel("Done")
        else:
            label = QLabel("ERROR: " + funcReturn)

        label.setFont(QFont('SansSerif', 10))
        label.setAlignment(Qt.AlignCenter)

        vBox.addWidget(label)

        dialog.setWindowTitle("Result")
        dialog.setLayout(vBox)
        dialog.exec_()

    ################################################################################################################################################################################

if __name__ == '__main__':
    gui = Gui()
    gui.show()
    sys.exit(app.exec_())
